@@ -3,10 +3,12 @@ import json
 import random
 import requests
 import math
-
 from typing import List, Dict, Optional
+
+from tqdm import tqdm
 from loguru import logger
 
+
 class Connection:
     def __init__(self, org_id: int):
         self._initialize(org_id)
@@ -66,9 +68,7 @@ class Connection:
     def _get_request(
         self, level: str, params: dict, call_uuid: Optional[str] = ""
     ) -> Dict:
-
         headers = {"Authorization": "Bearer %s" % self.access_token}
-
         if level == "call":
             url = os.getenv("CALL_URL")
         elif level == "turn":
@@ -76,23 +76,27 @@ class Connection:
                 raise AttributeError(
                     "Call UUID needs to be assigned for turn level requests"
                 )
-            url = os.getenv("TURN_URL").format(call_uuid)
+            url = os.getenv("TURN_URL")
         else:
             raise NotImplementedError(
                 "Currently only call and turn level get requests are supported"
             )
 
-        response = requests.get(url, headers=headers, params=params)
+        if url:
+            url = url.format(call_uuid)
+            try:
+                response = requests.get(url, headers=headers, params=params)
+                if response.status_code != 200:
+                    print(
+                        f"Status code {response.status_code} returned. text: {response.text}"
+                    )
 
-        if response.status_code != 200:
-            print(f"Status code {response.status_code} returned. text: {response.text}")
-
-        try:
-            return response.json()
-        except json.decoder.JSONDecodeError:
-            raise RuntimeError(
-                f"Status code {response.status_code} returned. text: {response.text}"
-            )
+                return response.json()
+            except json.decoder.JSONDecodeError:
+                raise RuntimeError(
+                    f"Status code {response.status_code} returned. text: {response.text}"
+                )
+        raise ValueError("Environment variable $TURN_URL was not set.")
 
     def fetch_calls(
         self,
@@ -101,13 +105,17 @@ class Connection:
         end: str,
         call_quantity: int,
         flow_uuid: Optional[str] = None,
-        page_size: int = 20
+        page_size: int = 20,
+        custom_search_key: Optional[str] = None,
+        custom_search_value: Optional[str] = None,
+        subtesting: bool = False,
     ) -> List:
         params = {
             "start": start,
             "end": end,
             "lang_code": lang_code,
             "page_size": page_size,
+            "call_type": "subtesting" if subtesting else "live",
             "ignored_caller_numbers": ["0000000000", "ev-connect"],
         }
 
@@ -116,6 +124,10 @@ class Connection:
         if flow_uuid:
             params["flow_uuid"] = flow_uuid
 
+        if custom_search_key:
+            params["custom_search_key"] = custom_search_key
+            params["custom_search_value"] = custom_search_value
+
         # Call Report API is Paginated so data is saved in pages, each page will be a get request
         logger.info(params)
 
@@ -124,12 +136,14 @@ class Connection:
         total_pages = first_page.get("total_pages", 2)
         call_summary = first_page.get("items", [])
         if PAGE_SCAN_LIMIT < total_pages:
-            pages_to_read = random.sample(range(current_page, total_pages), PAGE_SCAN_LIMIT)
+            pages_to_read = random.sample(
+                range(current_page, total_pages), PAGE_SCAN_LIMIT
+            )
         else:
             pages_to_read = list(range(current_page, total_pages))
-        logger.info(f"Reading random {pages_to_read} pages.")
+        logger.info(f"Reading random {len(pages_to_read)} pages.")
 
-        for page in pages_to_read:
+        for page in tqdm(pages_to_read):
             params["page"] = page
             logger.info(params)
             items = self._get_request("call", params).get("items", [])
